/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2013 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.actor.impl.io.fileload;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import java.io.IOException;
import java.io.FileInputStream;
import java.nio.ByteBuffer;
import java.nio.file.Files;
import java.nio.file.Path;
import it.tidalwave.util.Task;
import it.tidalwave.actor.annotation.Actor;
import it.tidalwave.messagebus.annotation.ListensTo;
import it.tidalwave.actor.io.filescan.FileDamageDetectedMessage;
import it.tidalwave.actor.io.fileload.FileContentsAvailableMessage;
import it.tidalwave.actor.io.fileload.FileLoadRequest;
import lombok.Cleanup;
import lombok.extern.slf4j.Slf4j;
import static java.nio.channels.FileChannel.MapMode.READ_ONLY;

/***********************************************************************************************************************
 * 
 * @stereotype Actor
 * 
 * An actor for loading the contents of a file.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Actor(initialPriority=Thread.MAX_PRIORITY) @ThreadSafe @Slf4j
public class FileLoaderActor
  {
    private final FileLoaderActorStats stats = new FileLoaderActorStats();
    
    void onFileLoadRequest (final @ListensTo @Nonnull FileLoadRequest message) 
      {
        final Path path = message.getPath();

        try 
          {
            final ByteBuffer byteBuffer = stats.execute(new Task<ByteBuffer, IOException>() 
              {
                @Override public ByteBuffer run() throws IOException
                  {
                    final @Cleanup FileInputStream fis = new FileInputStream(path.toFile());
                    return fis.getChannel().map(READ_ONLY, 0, Files.size(path)).load();
                  }
              });

            FileContentsAvailableMessage.forPath(path)
                                        .withContent(byteBuffer)
                                        .withLoadTime(stats.getLatestElapsedTime())
                                        .send();
            log.debug(">>>> {}", stats);
          }
        catch (IOException e) 
          {
            log.warn(e.toString());
            FileDamageDetectedMessage.forPath(path).withCause(e).send();
          }
        catch (Exception e) 
          {
            log.warn(e.toString());
            FileDamageDetectedMessage.forPath(path).withCause(e).send();
          }
      }
  }
